package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.SDONBuilder;
import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * The root object of an SDON document page.
 */
public class Document {
    /**
     * Type of diagram SDON will create. Sets the behavior of lines and shapes.
     * A value from the {@link SDONTemplates} enum.
     */
    @SDONSerializable
    public String Template = null;
    
    /**
     * Type of diagram SDON will create. Sets the behavior of lines and shapes.
     * A value from the {@link SDONTemplates} enum.
     * @param template - Template name
     * @return Document
     */
    public Document SetTemplate(String  template) {
    	Template = template;
    	
    	return this;
    }
    
    /**
     * The version of the SDON document.
     */
    @SDONSerializable
    public String Version = "20";
    
    
    /**
     * The root shape in a diagram.
     */
    @SDONSerializable
    public Shape Shape = new Shape();
    
    /**
     * Get the "main" or "root" shape of the document
     * @return Shape
     */
    public Shape GetTheShape(  ) {    	
    	return Shape;
    }
    
    /**
     * A title string centered over the diagram 1/2" above it.
     */
    @SDONSerializable
    public Title Title = null;
    
    /**
     * A title string centered over the diagram 1/2" above it.
     * @param title - Label of title
     * @return Title
     */
    public Title SetTitle(String title) {
    	Title = new Title();
    	
    	Title.SetLabel(title);
    	
    	return Title;
    }
    
    /**
     * A list of segmented lines that link shapes together.
     */
    @SDONSerializable
    public ArrayList<Return> Returns = null;
    
    /**
     * Add a Return. A segmented line that link shapes together.
     * @param startID - ID of shape at which to start line
     * @param endID - ID of shape at which to end line
     * @return Return
     */
    public Return AddReturn(int startID, int endID) {
    	if (Returns == null)
    		Returns = new ArrayList<Return>();
    	
    	Return ret = new Return();
    	ret.StartID = startID;
    	ret.EndID = endID;
    	
    	Returns.add(ret);
    	
    	return ret;
    }
    
    /**
     * A list of mappings of color aliases to color hex codes so that the alias
     * can be used instead of the hex color code.
     */
    @SDONSerializable
    public ArrayList<ColorEntry> Colors = null;
    
    /**
     * A list of mappings of color aliases to color hex codes so that the alias
     * can be used instead of the hex color code.
     * @param colors - List of color entries
     * @return
     */
    public Document SetColors(ArrayList<ColorEntry> colors) {
    	Colors = colors;
    	
    	return this;
    }
    
    /**
     * A list of mapping of a symbol's ID (in the SmartDraw CMS) to a alias
     * that can be used in place of the Shape's ShapeType name.
     */
    @SDONSerializable
    public ArrayList<SymbolEntry> Symbols = null;
    
    /**
     * A list of mapping of a symbol's ID (in the SmartDraw CMS) to a alias
     * that can be used in place of the Shape's ShapeType name.
     * @param symbols - List of symbol entries
     * @return Document
     */
    public Document SetSymbols(ArrayList<SymbolEntry> symbols) {
    	Symbols = symbols;
    	
    	return this;
    }
    
    /**
     * Definitions of data tables used in the diagram whose rows can be
     * referenced by a shape.
     */
    @SDONSerializable
    public ArrayList<DataTable> DataTable = null;
    
    /**
     * Add a data table.  Data tables contain information that can be associated with a shape
     * @param tableName - Name of table
     * @param tableID - ID of table
     * @return DataTable
     */
    public DataTable AddDataTable(String tableName, String tableID) {
    	if (DataTable == null)
    		DataTable = new ArrayList<DataTable>();
    	
    	DataTable dataTable = new DataTable();
    	
    	dataTable.ID = tableID;
    	dataTable.TableName = tableName;
    			
    	DataTable.add(dataTable);
    	
    	return dataTable;
    }
    
    /**
     * Get a table by its id
     * @param id - ID of table to get
     * @return DataTable
     */
    public DataTable GetDataTable(String id) {
    	if (DataTable == null) 
    		return null;
    	
    	for (int i = 0; i < DataTable.size(); i++) {
    		if (DataTable.get(i).ID == id)
    			return DataTable.get(i);
    	}
    	
    	return null;
    }
    
    /**
     * Document-wide defaults
     */
    @SDONSerializable
    public DocumentDefaults DocumentDefaults = null;
    
    /**
     * Set default horizontal spacing for connectors
     * @param horizontalSpacing - horizontal spacing amount
     * @return Document
     */
    public Document SetConnectorHorizontalSpacing(double horizontalSpacing) {
    	if (DocumentDefaults == null) 
    		DocumentDefaults = new DocumentDefaults();
    	
    	DocumentDefaults.HorizontalSpacing = horizontalSpacing;
    	
    	return this;
    }
    
    /**
     * Set default vertical spacing for connectors
     * @param verticalSpacing - vertical spacing amount
     * @return Document
     */
    public Document SetConnectorVerticalSpacing(double verticalSpacing) {
    	if (DocumentDefaults == null) 
    		DocumentDefaults = new DocumentDefaults();
    	
    	DocumentDefaults.VerticalSpacing = verticalSpacing;
    	
    	return this;
    }      
    
    /**
     * Return the JSON representation of this document
     * @return JSON representation of document
     */
    public String toJSON() {
    	return SDONBuilder.<Document>ToJSON(this);
    }
    
    /**
     * Return a Document from a SDON string (JSON)
     * @param sdonStr - string of sdon
     * @return Document
     */
    public static Document fromJSON(String sdonStr) {
    	return SDONBuilder.FromJSON(sdonStr);
    }

}